
// $Id: TaskAttributes.cpp,v 1.24 2009/09/09 02:29:47 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file Taskcc
  \brief stores TaskAttributes informations
  \author Hugo Pereira
  \version $Revision: 1.24 $
  \date $Date: 2009/09/09 02:29:47 $
*/

#include <QPixmap>

#include "Command.h"
#include "CustomPixmap.h"
#include "File.h"
#include "Icons.h"
#include "Str.h"
#include "TaskAttributes.h"
#include "XmlDef.h"
#include "XmlOptions.h"
#include "XmlTimeStamp.h"
#include "XmlString.h"

using namespace std;

//___________________________________________________
unsigned int& TaskAttributes::_idCount( void )
{
  static unsigned int counter(0);
  return counter;
}

//___________________________________________
TaskAttributes::TaskAttributes( void ):
  Counter( "TaskAttributes" ),
  id_( _idCount()++ ),
  flags_( NONE )
{
  Debug::Throw( "TaskAttributes::TaskAttributes\n" );
  setCreation();
  setIconName();
}

//___________________________________________
TaskAttributes::TaskAttributes( const QDomElement& element ):
  Counter( "TaskAttributes" ),
  id_( _idCount()++ ),
  flags_( NONE )
{
  Debug::Throw( "TaskAttributes::TaskAttributes\n" );

  QString arguments;

  // parse attributes
  QDomNamedNodeMap attributes( element.attributes() );
  for( unsigned int i=0; i<attributes.length(); i++ )
  {
    QDomAttr attribute( attributes.item( i ).toAttr() );
    if( attribute.isNull() ) continue;
    QString name( attribute.name() );
    QString value( attribute.value() );

    if( name == XML::NAME ) setName( XmlString( value ).toText() );
    else if( name == XML::COMMAND ) setCommand( XmlString( value ).toText() );
    else if( name == XML::ARGUMENTS ) arguments = XmlString( value ).toText();
    else if( name == XML::ICON ) setIconName( value );
    else if( name == XML::FLAGS ) setFlags( value.toInt() );
    else Debug::Throw(0) << "TaskAttributes::TaskAttributes - unrecognized name " << name << endl;
  }

  // parse children elements
  for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
  {
    QDomElement child_element = child_node.toElement();
    QString tag_name( child_element.tagName() );
    if( tag_name == XML::COMMENTS ) setComments( XmlString( child_element.text() ).toText() );
    else if( tag_name == XML::MODIFICATION ) setModification( XmlTimeStamp( child_element ) );
    else if( tag_name == XML::CREATION ) setCreation( XmlTimeStamp( child_element ) );
    else if( tag_name == XML::LAST_CALL ) setLastCalled( XmlTimeStamp( child_element ) );
    else Debug::Throw(0) << "TaskAttributes::TaskAttributes - unrecognized child " << child_element.tagName() << ".\n";
  }

  // add optional arguments to command
  // this is necessary to handle old arguments format
  if( !arguments.isEmpty() ) setCommand( command() + " " + arguments );

}

//___________________________________________
QDomElement TaskAttributes::domElement( QDomDocument& parent ) const
{

  Debug::Throw( "TaskAttributes::DomElement.\n");
  QDomElement out( parent.createElement( XML::TASK ) );
  out.setAttribute( XML::FLAGS, Str().assign<unsigned int>( flags() ) );
  if( flag( IS_SEPARATOR ) ) return out;

  // attributes
  if( !name().isEmpty() ) out.setAttribute( XML::NAME, XmlString( name() ).toXml() );
  if( !command().isEmpty() ) out.setAttribute( XML::COMMAND, XmlString( command() ).toXml() );
  if( !iconName().isEmpty() ) out.setAttribute( XML::ICON, iconName() );

  // comments
  if( !comments().isEmpty() )
  {
    QDomElement comments_element = parent.createElement( XML::COMMENTS );
    QDomText comments_text = parent.createTextNode( XmlString( comments() ).toXml() );
    comments_element.appendChild( comments_text );
    out.appendChild( comments_element );
  }

  // timestamps
  if( creation().isValid() )
  {

    Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " creation: " << creation().toString() << " " << creation() << endl;
    out.appendChild( XmlTimeStamp( creation() ).domElement( XML::CREATION, parent ) );
  }

  if( modification().isValid() )
  {
    Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " modification: " << modification().toString() << " " << modification() << endl;
    out.appendChild( XmlTimeStamp( modification() ).domElement( XML::MODIFICATION, parent ) );
  }

  if( lastCalled().isValid() )
  {
    Debug::Throw() << "TaskAttributes::DomElement - name: " << name() << " lastCalled: " << lastCalled().toString() << " " << lastCalled() << endl;
    out.appendChild( XmlTimeStamp( lastCalled() ).domElement( XML::LAST_CALL, parent ) );
  }
  return out;

}

//___________________________________________
void TaskAttributes::setIconName( const QString& value )
{

  if( CustomPixmap( value ).isNull() ) icon_filename_ = defaultIconFilename();
  else icon_filename_ = value;
  return;

}

//___________________________________________
QString TaskAttributes::defaultIconFilename( void ) const
{
  Debug::Throw( "TaskAttributes::defaultIconFilename.\n" );

  // set to (truncated) default name
  File file( ICONS::DEFAULT );
  QString out( ICONS::DEFAULT );

  // load icon path list from options
  Options::List path_list( XmlOptions::get().specialOptions( "PIXMAP_PATH" ) );
  if( path_list.empty() ) return out;

  // loop over path and find matching pixmaps
  for( Options::List::const_iterator iter = path_list.begin(); iter != path_list.end(); iter++ )
  {

    // skip empty path
    if( iter->raw().isEmpty() ) continue;

    // prepare filename
    File icon_file;

    // see if path is internal resource path
    if( iter->raw()[0] == ':' ) icon_file = file.addPath( iter->raw() );
    else icon_file = File( iter->raw() ).find( file, true );

    // load pixmap
    if( !( icon_file.isEmpty() && QPixmap( icon_file ).isNull() ) )
    {
      out = icon_file;
      break;
    }
  }

  Debug::Throw() << "TaskAttributes::defaultIconFilename - filename: " << out << endl;
  return out;

}

//__________________________________________________
bool TaskAttributes::exec( void )
{

  Debug::Throw( "TaskAttributes::exec.\n" );

  // check if separator
  if( flag( IS_SEPARATOR ) ) return true;

  setLastCalled();
  if( command().isEmpty() ) return false;
  else return Command( command() ).run();
}
