// $Id: TaskBarFixed.cpp,v 1.30 2009/09/09 02:29:48 hpereira Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file    TaskBarFixed.cc
  \brief   transparent frame to display TaskButton(s)
  \author  Hugo Pereira
  \version $Revision: 1.30 $
  \date    $Date: 2009/09/09 02:29:48 $
*/

#include <algorithm>

#include "SvgEngine.h"
#include "TaskBarFixed.h"
#include "XmlOptions.h"

#include "CompositeEngine.h"

using namespace std;

//____________________________________________________
TaskBarFixed::TaskBarFixed( QWidget* parent ):
  TaskBarBase( parent )
{ Debug::Throw( "TaskBarFixed::TaskBarFixed.\n" ); }

//____________________________________________________________________
void TaskBarFixed::_updateBackgroundPixmap( void )
{

  TaskBarBase::_updateBackgroundPixmap();
  if( _useSvg() && SVG::SvgEngine::get().isValid() )
  {
    QPainter painter( &_backgroundPixmap() );
    painter.setRenderHints(QPainter::SmoothPixmapTransform);
    painter.setCompositionMode(QPainter::CompositionMode_Source );
    painter.drawPixmap( QPoint(0,0),  SVG::SvgEngine::get().get( size() ) );
  }


}

//________________________________________________
void TaskBarFixed::_resize( void )
{
  Debug::Throw( "TaskBarFixed::_resize.\n" );

  // if no tasks, set empty message and exit
  if( _buttons().empty() )
  {
    resize( emptySize() );
    return;
  }

  // dimensions
  _setDimensions();

  // retrieve max value of buttons minimum size
  QSize button_size(
    (*max_element(  _buttons().begin(), _buttons().end(),  TaskButton::MaxWidthFTor() ))->maximumSize().width(),
    (*max_element(  _buttons().begin(), _buttons().end(),  TaskButton::MaxHeightFTor() ))->maximumSize().height()
  );

  // compute sizes and set TaskButtons
  unsigned int height(0);
  unsigned int width(0);

  // loop over buttons, compute the minimum size needed
  const TaskBarAttributes::Layout &layout( attributes().layout() );
  if( layout & TaskBarAttributes::HORIZONTAL )
  {

    /*
      for horizontal layout, the height is fixed to the largest button min height
      whereas the width is the current button min width
    */
    height = button_size.height()+2*_margin();
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    { width += (*iter)->maximumSize().width()+_spacing(); }

    width += 2*_margin() - _spacing();

  } else if( layout & TaskBarAttributes::VERTICAL ) {

    /*
      for vertical layout, the height is the current button min height
      whereas the width is fixed to the largest button min width
    */
    width =  button_size.width()+2*_margin();
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    { height += (*iter)->maximumSize().height()+_spacing(); }

    height += 2*_margin() - _spacing();

  } else if( layout & TaskBarAttributes::BOX ) {

    /*
      for box layout, the width and height are fixed to the largest button min width/height
    */

    width = _numColumns()*(button_size.width()+_spacing() ) + 2*_margin() - _spacing();
    height = _numLines()*(button_size.height()+_spacing() ) + 2*_margin() - _spacing();

  } else assert(0);

  // resize
  // Debug::Throw(0) << "TaskBarFixed::_resize - (" << width << "," << height << ")" << endl;
  resize( width, height );

}


//______________________________________________________
void TaskBarFixed::_placeButtons( void )
{
  Debug::Throw( "TaskBarFixed::_placeButtons.\n" );

  // retrieve max value of buttons minimum size
   QSize button_size(
    (*max_element(  _buttons().begin(), _buttons().end(),  TaskButton::MaxWidthFTor() ))->maximumSize().width(),
    (*max_element(  _buttons().begin(), _buttons().end(),  TaskButton::MaxHeightFTor() ))->maximumSize().height()
  );

  int x( _margin() );
  int y( _margin() );

  // loop over buttons, compute the minimum size needed
  const TaskBarAttributes::Layout &layout( attributes().layout() );
  if( layout & TaskBarAttributes::HORIZONTAL )

  /*
    for horizontal layout, the height is fixed to the largest button min height
    whereas the width is the current button min width
  */
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    (*iter)->setRect( QPoint( x, y ), QSize( (*iter)->maximumSize().width()+_spacing(), button_size.height()+_spacing() ) );
    x+=(*iter)->rect().width();
  }

  else if( layout & TaskBarAttributes::VERTICAL )

  /*
    for vertical layout, the height is the current button min height
    whereas the height is fixed to the largest button min width
  */
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    (*iter)->setRect( QPoint( x, y ), QSize( button_size.width()+_spacing(), (*iter)->maximumSize().height()+_spacing() ) );
    y += (*iter)->rect().height();
  }

  else if( layout & TaskBarAttributes::BOX )

  /*
    for box layout, the width and height are fixed to the largest button min width/height
  */
  // get number of lines
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    (*iter)->setRect( QPoint( x, y ), QSize(button_size.width()+_spacing(), button_size.height()+_spacing() ) );
    x+=(*iter)->rect().width();
    if( int(x) >= width() )
    {
      x = _margin();
      y+=(*iter)->rect().height();
    }
  }

  else assert(0);

}

//______________________________________________________
bool TaskBarFixed::_drawButtons( QPaintDevice& device, QRect rect )
{

  Debug::Throw( "TaskBarFixed::_drawButtons.\n" );

  // one should draw directly on the widget
  // do nothing
  if( _buttons().empty() ) return false;

  // erase all buttons
  QPainter painter( &device );
  painter.setClipRect( rect );

  if( TRANSPARENCY::CompositeEngine::get().isEnabled() )
  {
    painter.setRenderHints(QPainter::SmoothPixmapTransform);
    painter.setCompositionMode(QPainter::CompositionMode_Source );
  }

  painter.translate( _offset() );
  painter.drawPixmap( QPoint(0,0),  _backgroundPixmap() );

  // place the buttons
  _placeButtons();

  painter.setCompositionMode(QPainter::CompositionMode_SourceOver );

  // redraw
  bool out( false );
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {

    (*iter)->paint( painter );
    if( _needUpdate() ||  (*iter)->direction() != TaskButton::NONE || (*iter)->highlightDirection() != TaskButton::NOHIGHLIGHT)
    {
      if( _action() != ENTER )
      {
        (*iter)->updateDirection();
        (*iter)->updateHighlightDirection();
        out |= ((*iter)->direction() != TaskButton::NONE);
        out |= (*iter)->highlightDirection() != TaskButton::NOHIGHLIGHT;
      }

    }

  }

  painter.end();

  // reset need_update state
  _setNeedUpdate( false );

  return out;

}
