#ifndef _TaskButton_h_
#define _TaskButton_h_

// $Id: TaskButton.h,v 1.26 2010/04/23 22:02:00 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    TaskButton.h
\brief   Taskbar button
\author  Hugo Pereira
\version $Revision: 1.26 $
\date    $Date: 2010/04/23 22:02:00 $
*/

#include <QColor>
#include <QPainter>
#include <QPixmap>
#include <QTimer>

#include <vector>

#include "Counter.h"
#include "PixmapList.h"
#include "TaskAttributes.h"

//! taskbar button
class TaskButton: public QObject, public Counter
{

    public:

    //! shortcut to list of buttons
    typedef TaskButton* Pointer;
    typedef std::vector<Pointer> List;

    //! null pointer
    static Pointer null( void )
    { return 0; }

    //! constructor
    TaskButton( const TaskBarAttributes& bar_attributes, const TaskAttributes& attributes ):
        Counter( "TaskButton" ),
        needResize_( false ),
        showName_( false ),
        textSize_( 0, 0 ),
        shadowOffset_( 0 ),
        highlight_( false ),
        maximumHighlightIndex_( 0 ),
        highlightIndex_( 0 ),
        true_highlightIndex_( 0 ),
        highlightDirection_( NOHIGHLIGHT ),
        index_( 0 ),
        direction_( NONE ),
        valid_( false )
    {
        Debug::Throw( "TaskButton::TaskButton.\n" );
        setAttributes( bar_attributes, attributes );
    }

    //! destructor
    virtual ~TaskButton( void )
    { Debug::Throw( "TaskButton::~TaskButton.\n" ); }

    //! equal to operator
    bool operator == ( const TaskButton& button ) const
    { return attributes().id() == button.attributes().id(); }

    //! true if button is valid
    const bool& isValid( void ) const
    { return valid_; }

    //! Rect
    void setRect( const QRect& rect )
    { rect_ = rect; }

    //! Rect
    void setRect( const QPoint& point, const QSize& size )
    { rect_ = QRect( point, size ); }

    //! Rect
    const QRect& rect( void ) const
    { return rect_; }

    //! Rect
    QRect& rect( void )
    { return rect_; }

    //! size of pixmap for index id
    QSize size( const unsigned int& index ) const;

    //! maximum size
    QSize minimumSize( void ) const
    { return size(0); }

    //! current size
    QSize currentSize( void ) const
    { return size( index_ ); }

    //! maximum size
    QSize maximumSize( void ) const
    { return size( maximumIndex()-1 ); }

    //! text size
    const QSize& textSize( void ) const
    { return textSize_; }

    //! attributes
    void setAttributes( const TaskBarAttributes& bar_attributes, const TaskAttributes& attributes );

    //! attributes
    const TaskAttributes& attributes( void ) const
    { return attributes_; }

    //! attributes
    TaskAttributes& attributes( void )
    { return attributes_; }

    //! Need resize
    const bool& needResize( void ) const
    { return needResize_; }

    //@name highlight
    //@{

    bool hasHighlightedPixmaps( void ) const
    { return pixmaps_.maxHighlightIndex() > 1; }

    //! highlight direction
    enum HighlightDirection {
        LIGHTER,
        DARKER,
        NOHIGHLIGHT
    };

    //! highlight
    void highlight( const bool& value )
    {
        highlight_ = value;
        if( highlight_ ) highlightDirection_ = LIGHTER;
    }

    //! update highlight direction
    const HighlightDirection& highlightDirection( void ) const
    { return highlightDirection_; }

    //! update highlight direction
    const HighlightDirection& updateHighlightDirection( void );

    //! highlight index
    const unsigned int& highlightIndex( void ) const
    { return true_highlightIndex_; }

    //@name zoom
    //@{

    //! zoom direction
    enum Direction {
        BIGGER,
        SMALLER,
        NONE
    };

    //! zoom direction
    void setDirection( const Direction& );

    //! zoom direction
    const Direction& direction( void ) const
    { return direction_; }

    //! max index (biggest zoom)
    const unsigned int& maximumIndex( void ) const
    { return pixmaps_.maxSizeIndex(); }

    //! current index
    const unsigned int& index( void ) const
    { return index_; }

    //! current index
    void setIndex( const unsigned int& index )
    {
        assert( index < maximumIndex() );
        index_ = index;
    }

    //! update direction
    /*! direction is changed when button is at maximum size */
    const Direction& updateDirection( void );

    //@}

    //! erase
    /*! fill the button rect() geometry with the background pixmap */
    void erase( QPainter& painter, QPixmap background ) const
    { painter.drawPixmap( rect_.topLeft(), background, rect_ ); }

    //! paint
    /*
    redraw itself at proper position in painter
    \param painter, the destination painter
    the current index is used to choose pixmap.
    It is updated according to the button zoom direction.
    */
    void paint( QPainter& painter ) const;

    //! used to retrieve a button from point position
    class ContainsFTor
    {

        public:

        //! constructor
        ContainsFTor( const QPoint& point ):
            point_( point )
        {}

        //! predicate
        bool operator() (const TaskButton& button) const
        { return button.rect().contains( point_ ); }

        //! predicate
        bool operator() (const Pointer& button) const
        { return operator()( *button ); }

        private:

        //! prediction point
        const QPoint& point_;

    };

    //! used to retrieve a button from attributes
    class SameAttributesFTor
    {

        public:

        //! constructor
        SameAttributesFTor( const TaskAttributes& attributes ):
            attributes_( attributes )
        {}

        //! predicate
        bool operator() (const TaskButton& button ) const
        { return button.attributes().id() == attributes_.id(); }

        //! predicate
        bool operator() (const Pointer& button) const
        { return operator()( *button ); }

        private:

        //! predicted attributes
        const TaskAttributes& attributes_;

    };

    //! used to get button of maximum width
    class MaxWidthFTor
    {

        public:

        //! predicate
        bool operator() (const TaskButton& first, const TaskButton& second ) const
        { return first.maximumSize().width() < second.maximumSize().width(); }

        //! predicate
        bool operator() (const Pointer& first, const Pointer& second ) const
        { return operator()( *first, *second ); }

    };

    //! used to get button of maximum height
    class MaxHeightFTor
    {

        public:

        //! predicate
        bool operator() (const TaskButton& first, const TaskButton& second ) const
        { return first.maximumSize().height() < second.maximumSize().height(); }

        //! predicate
        bool operator() (const Pointer& first, const Pointer& second ) const
        { return operator()( *first, *second ); }

    };

    //! used to get button of minimum width
    class MinWidthFTor
    {

        public:

        //! predicate
        bool operator() (const TaskButton& first, const TaskButton& second ) const
        { return first.minimumSize().width() < second.minimumSize().width(); }

        //! predicate
        bool operator() (const Pointer& first, const Pointer& second ) const
        { return operator()( *first, *second ); }

    };

    //! used to get button of minimum height
    class MinHeightFTor
    {

        public:

        //! predicate
        bool operator() (const TaskButton& first, const TaskButton& second ) const
        { return first.minimumSize().height() < second.minimumSize().height(); }

        //! predicate
        bool operator() (const Pointer& first, const Pointer& second ) const
        { return operator()( *first, *second ); }

    };

    //! used to get the button with the maximum number of mixmaps
    class MaxIndexFTor
    {
        public:

        //! predicate
        bool operator() (const TaskButton& first, const TaskButton& second ) const
        {
            if( first.attributes().flag( TaskAttributes::IS_SEPARATOR ) ) return false;
            if( second.attributes().flag( TaskAttributes::IS_SEPARATOR ) ) return true;
            return first.maximumIndex() < second.maximumIndex();
        }

        //! predicate
        bool operator() (const Pointer& first, const Pointer& second ) const
        { return operator()( *first, *second ); }

    };

    private:

    //! need resize
    void _setNeedResize( const bool& value )
    { needResize_ = value; }

    //! validity
    void _setIsValid( const bool& validity )
    { valid_ = validity; }

    //! draw task pixmap in painter
    void _drawPixmap( QPainter& painter, const unsigned int& index ) const;

    //! draw task name in painter
    void _drawText( QPainter& painter, const unsigned int& index ) const;

    //! update highlight index
    void _updateTrueHighlightIndex( void );

    //! pixmaps
    const PixmapList& _pixmaps( void ) const
    { return pixmaps_; }

    //! pixmaps
    PixmapList& _pixmaps( void )
    { return pixmaps_; }

    //! associated task
    TaskAttributes attributes_;

    //!@name geometry
    //@{

    //! pixmap position in painter
    QRect rect_;

    //! true if new maximumSize is bigger after SetAttributes
    bool needResize_;

    //@}

    //!@name button name
    //!@{

    //! true if name is to be drawn
    bool showName_;

    //! text size
    QSize textSize_;

    //! shadow offset
    int shadowOffset_;

    //! foreground color
    QColor foregroundColor_;

    //! shadow color
    QColor shadowColor_;

    //@}

    //! vector of scaled pixmap from small to high
    PixmapList pixmaps_;

    //! true if highlighted
    bool highlight_;

    //! maximum highlightIndex_
    unsigned int maximumHighlightIndex_;

    //! highlight index
    double highlightIndex_;

    //! highlight index
    unsigned int true_highlightIndex_;

    //! highlight direction
    HighlightDirection highlightDirection_;

    //! current position in pixmap vector
    unsigned int index_;

    //! zoom direction
    Direction direction_;

    //! validity
    bool valid_;

};

#endif
