// $Id: TaskToolTip.cpp,v 1.28 2010/07/22 22:03:39 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    TaskToolTip.cc
\brief   application splash screen
\author  Hugo Pereira
\version $Revision: 1.28 $
\date    $Date: 2010/07/22 22:03:39 $
*/

#include <QLayout>
#include <QPainter>

#include "IconSize.h"

#include "CustomPixmap.h"
#include "Debug.h"
#include "Effects.h"
#include "Options.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "SvgEngine.h"
#include "TaskToolTip.h"
#include "XmlOptions.h"
#include "X11Util.h"

#include "CompositeEngine.h"

using namespace std;

static unsigned int debug_level(1);

//_______________________________________________________________________
TaskToolTip::TaskToolTip( QWidget* parent, const TaskAttributes& attributes ):
    TRANSPARENCY::TransparentWidget( parent, Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint ),
    delay_( 0 ),
    flags_( NONE ),
    pixmapSize_( IconSize( IconSize::MAXIMUM ) ),
    margin_( 10 ),
    useSvg_( false ),
    shadowOffset_( 0 )
{

    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();

    // attributes
    setAttributes( attributes );

}

//__________________________________________________
void TaskToolTip::setAttributes( const TaskAttributes& attributes )
{

    // try load pixmap from cache
    Cache::const_iterator iter( cache_.find( attributes.iconName() ) );
    if( iter != cache_.end() ) _setPixmap( iter->second );
    else {

        // if not found, create a new pixmap,
        CustomPixmap pixmap( attributes.iconName() );
        if( pixmap.isNull() ) pixmap = CustomPixmap().empty( pixmapSize_ );
        else pixmap = pixmap.scaled( pixmapSize_, Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // insert to pixmap
        cache_.insert( make_pair( attributes.iconName(), pixmap ) );

        // assign to local
        _setPixmap( pixmap );

    }

    // title and comment
    _setTitle( attributes.name() );
    _setComments( attributes.comments() );

    // update tooltip size
    _adjustSize();

    // redraw if needed
    // if( isVisible() ) update();

}

//____________________________________
void TaskToolTip::show( void )
{
    Debug::Throw( debug_level, "TaskToolTip::Show.\n" );
    stopTimers();
    if( !flag( VISIBLE ) ) _showTimer().start( delay_, this );
    else {
        setFlag( VISIBLE, true );
        QWidget::show();
        _hideTimer().start( 2*delay_, this );
    }

}

//____________________________________
void TaskToolTip::hide( bool now )
{

    Debug::Throw( debug_level, "TaskToolTip::Hide.\n" );
    stopTimers();
    QWidget::hide();
    if( now ) setFlag( VISIBLE, false );
    else _hideTimer().start( delay_, this );

}

//____________________________________
void TaskToolTip::stopTimers( void )
{

    Debug::Throw( debug_level, "TaskToolTip::StopTimers.\n" );
    _showTimer().stop();
    _hideTimer().stop();

}

//____________________________________
void TaskToolTip::_updateConfiguration( void )
{
    Debug::Throw( "TaskToolTip::_updateConfiguration.\n" );

    // rounded flags
    setFlag( ROUNDED, XmlOptions::get().get<bool>("ROUND_TOOLTIP") );

    // transparency
    bool transparent(  XmlOptions::get().get<bool>( "TRANSPARENT_TOOLTIP" ) );
    if( transparent )
    {

        _setTransparent( true );
        setWindowOpacity( XmlOptions::get().get<double>( "TRANSPARENCY_OPACITY" )/255 );

    } else {

        _setTransparent( false );
        _setTintColor( QWidget().palette().color( QPalette::Window ) );
        setWindowOpacity( 1 );

    }

    // update foreground color
    QColor foreground_color( XmlOptions::get().raw( "TRANSPARENCY_FOREGROUND_COLOR" ) );
    QPalette palette( TaskToolTip::palette() );
    if( foreground_color.isValid() )
    {
        palette.setColor( QPalette::Text, foreground_color );
        palette.setColor( QPalette::WindowText, foreground_color );
        setPalette( palette );
    }

    // fonts
    largeFont_ = font();
    largeFont_.setPointSize( 20 );
    largeFont_.setBold( true );

    normalFont_ = font();

    // pixmap size
    unsigned int size = (unsigned int)( XmlOptions::get().get<unsigned int>( "TOOLTIP_HEIGHT" ) );
    pixmapSize_ = QSize( size, size );


    // svg
    _setUseSvg( XmlOptions::get().get<bool>( "USE_SVG" ) );
    _setShadowOffset( XmlOptions::get().get<int>( "TRANSPARENCY_SHADOW_OFFSET" ) );
    _setShadowColor( XmlOptions::get().raw( "TRANSPARENCY_SHADOW_COLOR" ) );

    // margin
    margin_ = 10 + XmlOptions::get().get<int>( "MARGIN" );

    // works on X11 only
    X11Util::get().changeProperty( *this, X11Util::_NET_WM_STATE_SKIP_TASKBAR, true );
    X11Util::get().changeProperty( *this, X11Util::_NET_WM_STATE_SKIP_PAGER, true );

    return;
}

//__________________________________________________
void TaskToolTip::_paint( QPaintDevice& device, const QRect& rect )
{

    QPainter painter( &device );
    painter.setClipRect( rect );
    painter.setPen( palette().color( QPalette::Text ) );

    if( TRANSPARENCY::CompositeEngine::get().isEnabled() )
    {
        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
    }

    if( _useSvg() && SVG::SvgEngine::get().isValid() )
    { painter.drawPixmap( QPoint(0,0), SVG::SvgEngine::get().get( size() ) ); }

    painter.setCompositionMode(QPainter::CompositionMode_SourceOver );

    // pixmap
    if( !_pixmap().isNull() ) painter.drawPixmap( margin_, margin_, _pixmap() );

    QPixmap pixmap( size() );
    pixmap.fill( Qt::transparent );
    {
        QPainter painter( &pixmap );
        painter.setClipRect( rect );
        painter.setPen( palette().color( QPalette::Text ) );

        // font metrics
        QFontMetrics large_metrics( largeFont_ );
        QFontMetrics normal_metrics( normalFont_ );

        // title
        painter.setFont( largeFont_ );
        painter.drawText( 2*margin_ + _pixmap().width(), margin_ + large_metrics.lineSpacing(), _title() );

        // comments
        painter.setFont( normalFont_ );
        painter.drawText( 2*margin_ + _pixmap().width(), 2*margin_ + large_metrics.lineSpacing() + normal_metrics.lineSpacing(), _comments() );
        painter.end();
    }

    if( _shadowOffset() > 0 && _shadowColor().isValid() )
    {
        QImage image( pixmap.toImage() );
        TRANSPARENCY::Effects::shadowBlur(image, _shadowOffset(), _shadowColor() );
        painter.drawImage( TaskToolTip::rect().topLeft(), image);
    }

    painter.drawPixmap( TaskToolTip::rect().topLeft(), pixmap );

}

//__________________________________________________
void TaskToolTip::timerEvent( QTimerEvent* event )
{

    if( event->timerId() == _showTimer().timerId() )
    {

        setFlag( VISIBLE, true );
        QWidget::show();
        _showTimer().stop();
        _hideTimer().start( 2*delay_, this );

    } else if( event->timerId() == _hideTimer().timerId() ) {

        setFlag( VISIBLE, false );
        _hideTimer().stop();
        if( isVisible() ) QWidget::hide();

    } else return QWidget::timerEvent( event );

}

//__________________________________________________
void TaskToolTip::resizeEvent( QResizeEvent* )
{
    if( flag( ROUNDED ) ) setMask( RoundedRegion( rect(), RoundedRegion::ALL ) ); 
    if( _blurEnabled() ) {
        
        static const int margin = 10;
        _updateBlurRegion( rect().translated( mapTo( window(), QPoint(0,0) ) ).adjusted( margin, margin, -margin, -margin ) );
    
    }
}

//__________________________________________________
void TaskToolTip::_adjustSize( void )
{

    QFontMetrics large_metrics( largeFont_ );
    QFontMetrics normal_metrics( normalFont_ );

    int width = margin_ + _pixmap().width() + margin_ +
        std::max( large_metrics.width(_title()), normal_metrics.width( _comments() ) ) +
        margin_;

    int height = margin_ + std::max(
        _pixmap().height(),
        large_metrics.lineSpacing() + margin_ + normal_metrics.lineSpacing() ) +
        margin_;

    size_ = QSize( width, height );

}
